﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Trolleybus.Entities;

namespace Trolleybus.DrawingObjects
{
    public class DrawingTrolleybus : DrawingBus
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="horns">Признак наличия "рогов"</param>
        /// <param name="batteries">Признак наличия отсека под электрические батареи</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>
        public DrawingTrolleybus(int speed, double weight, Color bodyColor, Color additionalColor, bool horns, bool batteries, int width, int height) : base(speed, weight, bodyColor, width, height, 150, 95)
        {
            if (EntityBus != null)
            {
                EntityBus = new EntityTrolleybus(speed, weight, bodyColor, additionalColor, horns, batteries);
            }

        }
        public override void DrawTransport(Graphics g)
        {
            if (EntityBus is not EntityTrolleybus trolleyBus)
            {
                return;
            }        
            Pen additionalPen = new(trolleyBus.AdditionalColor);
            Brush additionalBrush = new SolidBrush(trolleyBus.AdditionalColor);
            base.DrawTransport(g);           
            //опциональные "рога"
            if (trolleyBus.Horns)
            {
                g.DrawLine(additionalPen, _startPosX + 70, _startPosY + 30, _startPosX + 40, _startPosY);
                g.DrawLine(additionalPen, _startPosX + 70, _startPosY + 30, _startPosX + 60, _startPosY);
            }

            //опциональный отсек для батареи
            if (trolleyBus.Batteries)
            {
                Point[] pointsOfBatteries = { new Point(_startPosX + 70, _startPosY + 30), new Point(_startPosX + 70, _startPosY + 25), new Point(_startPosX + 100, _startPosY + 25), new Point(_startPosX + 110, _startPosY + 30)};
                g.FillPolygon(additionalBrush, pointsOfBatteries);
            }
        }
    }
}

