﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Trolleybus.Entities;
using Trolleybus.DrawingObjects;
using System.Configuration;

namespace Trolleybus
{
    public static class ExtensionDrawingBus
    {
        /// <summary>
        /// Создание объекта из строки
        /// </summary>
        /// <param name="info">Строка с данными для создания объекта</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <param name="width">Ширина</param>
        /// <param name="height">Высота</param>
        /// <returns>Объект</returns>
        public static DrawingBus? CreateDrawingBus(this string info, char separatorForObject, int width, int height)
        {
            string[] strs = info.Split(separatorForObject);
            if (strs.Length == 3)
            {
                return new DrawingBus(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]), Color.FromName(strs[2]), width, height);
            }
            if (strs.Length == 6)
            {
                return new DrawingTrolleybus(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]),
                Color.FromName(strs[2]),
                Color.FromName(strs[3]),
                Convert.ToBoolean(strs[4]),
                Convert.ToBoolean(strs[5]),
                width, height);
            }
            return null;
        }

        /// <summary>
        /// Получение данных для сохранения в файл
        /// </summary>
        /// <param name="drawingBus">Сохраняемый объект</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <returns>Строка с данными по объекту</returns>
        public static string GetDataForSave(this DrawingBus drawingBus, char separatorForObject)
        {
            var bus = drawingBus.EntityBus;
            if (bus == null)
            {
                return string.Empty;
            }
            var str = $"{bus.Speed}{separatorForObject}{bus.Weight}{separatorForObject}{bus.BodyColor.Name}";
            //Если это автобус (т.е. не троллейбус), то строки из 3 свойств достаточно
            if (bus is not EntityTrolleybus trolleyBus)
            {
                return str;
            }
            //Если это троллейбус, то дополнение строки из 3 свойств доп. свойствами
            return $"{str}{separatorForObject}{trolleyBus.AdditionalColor.Name}{separatorForObject}{trolleyBus.Horns}{separatorForObject}{trolleyBus.Batteries}";
        }
    }
}

